#visualisation {
    position: relative;
}


.metagenomics-sankey h3 {
    font-size: 24px;
    margin: 0;
    padding: 15px 0 15px 0;
    font-family: inherit;
    font-weight: 500;
    line-height: 1.1;
}

.metagenomics-sankey body {
    height: 100%;
    padding: 0;
    margin: 0;
    box-sizing: border-box;

}

.metagenomics-sankey ul {
    padding: 0;
    margin: 0;
    list-style: none;
}

.metagenomics-sankey input,
.metagenomics-sankey select,
.metagenomics-sankey select.dataTable-selector {
    min-width: 200px;
    cursor: pointer;
    border: 0;
    padding: 0;
    margin: 0;
    position: relative;
    margin-top: 5px;
    outline: none;
    background-color: #0079a4;
    padding: 15px 25px;
    border: 2px solid #0079a4;
    color: white;
    text-transform: uppercase;
    font-size: 11px;
    border-radius: 4px;
    /* font-weight: bold; */
    letter-spacing: 0.05em;
    transition: 0.2s ease-in-out all;
    font-weight: bold;
    /* for Firefox */
    appearance: none;
    -moz-appearance: none;
    /* for Chrome */
    -webkit-appearance: none;
}

.metagenomics-sankey input {
    border: 2px solid #0079a4;
    background-color: white;
    color: #0079a4;
}

.metagenomics-sankey input::placeholder {
    color: #0079a4;
}

.metagenomics-sankey select:hover {
    background-color: #0079a4;
    color: white
}

.metagenomics-sankey option {
    color: black
}

.metagenomics-sankey button {
    white-space: nowrap;
    cursor: pointer;
    border: 0;
    padding: 0;
    margin: 0;
    position: relative;
    outline: none;
    background-color: transparent;
    padding: 15px 15px;
    border: 2px solid #0079a4;
    color: #0079a4;
    text-transform: uppercase;
    font-size: 11px;
    border-radius: 4px;
    /* font-weight: bold; */
    letter-spacing: 0.05em;
    transition: 0.2s ease-in-out all;
    font-weight: bold;
    /* for Firefox */
    appearance: none;
    -moz-appearance: none;
    /* for Chrome */
    -webkit-appearance: none;
}

.metagenomics-sankey button:hover {
    background-color: #0079a4;
    color: white
}

.metagenomics-sankey label {
    color: black;
    text-transform: uppercase;
    font-size: 11px;
    font-weight: normal;
    letter-spacing: 0.05em;
}

.metagenomics-sankey main {
    max-width: 1200px;
    /* padding: 50px; */
    margin: 0 auto;
}

.metagenomics-sankey #controls {
    max-width: 1200px;
}

.metagenomics-sankey #controls ul {
    display: flex;
}

.metagenomics-sankey #controls ul li {
    display: flex;
    flex-direction: column;
    margin: 0 15px 0 0;
    justify-content: end;
}
