"""Mulit-plot grids."""

__all__ = ['pairplot', 'jointplot']


def pairplot(
        *args, **kwargs):
    """Plot pairwise relationships in a dataset."""
    raise NotImplementedError


def jointplot(
        *args, **kwargs):
    """Draw a plot of two variables with bivariate and univariate graphs."""
    raise NotImplementedError
